#!/bin/bash

# defaults
profiles=/etc/pandora/conf/tvout-profiles.conf
tv_type="pal"
conn_type="svideo"
layer=0
pal_size="658,520"
pal_pos="35,35"
ntsc_size="655,455"
ntsc_pos="40,15"

usage()
{
  echo "usage: $0 [-d] [-t pal|ntsc] [-c composite|svideo] [-l 0|1|o] [-{p|n}s w,h] [-{p|n}p x,y]"
  exit 1
}

setup_scaler()
{
  # we must preallocate enough memory for the scaler layer
  # since the app won't be able to change this
  # the whole time TV-out is running..
  ofbset -fb /dev/fb1 -mem $[3*1024*1024] -size 512 256 -en 0
}

enable_it()
{
  size=$pal_size
  pos=$pal_pos
  if [ "$tv_type" = "ntsc" ]; then
    size=$ntsc_size
    pos=$ntsc_pos
  fi
  echo "${tv_type}, ${conn_type}, layer $layer, $pos $size"

  cd /sys/devices/platform/omapdss
  ovl0=`cat overlay0/enabled`
  ovl1=`cat overlay1/enabled`
  echo 0 > display1/enabled
  echo 0 > overlay0/enabled
  echo 0 > overlay1/enabled
  echo 0 > overlay2/enabled
  echo "" > /sys/class/graphics/fb2/overlays
  echo "" > /sys/class/graphics/fb1/overlays
  if [ $layer -eq 1 ]; then
    setup_scaler
    echo "0" > /sys/class/graphics/fb0/overlays
    echo "1,2" > /sys/class/graphics/fb1/overlays
  else
    # assume layer 0 for now
    echo "0,2" > /sys/class/graphics/fb0/overlays
    echo "1" > /sys/class/graphics/fb1/overlays
  fi
  echo $conn_type > display1/venc_type
  echo "tv" > overlay2/manager
  echo $tv_type > display1/timings
  echo $size > overlay2/output_size
  echo $pos > overlay2/position 
  echo $ovl0 > overlay0/enabled
  echo $ovl1 > overlay1/enabled
  echo 1 > overlay2/enabled
  echo 1 > display1/enabled
}

disable_it()
{
  cd /sys/devices/platform/omapdss
  echo 0 > overlay0/enabled
  echo 0 > overlay1/enabled
  echo 0 > overlay2/enabled
  echo 0 > display1/enabled
  echo "" > /sys/class/graphics/fb2/overlays
  echo "" > /sys/class/graphics/fb1/overlays
  echo 0 > /sys/class/graphics/fb0/overlays
  echo 1 > /sys/class/graphics/fb1/overlays
  # for system menu, etc
  echo 2 > /sys/class/graphics/fb2/overlays
  echo "lcd" > overlay2/manager
  echo 1 > overlay0/enabled
}

layer_toggle()
{
  tv_on=`cat /sys/devices/platform/omapdss/display1/enabled`
  if [ "$tv_on" != "1" ]; then
    echo "tv-out is off, not changing layers"
    exit 1
  fi
  ovl1_on=`cat /sys/devices/platform/omapdss/overlay1/enabled`
  if [ "$ovl1_on" != "1" ]; then
    layer=0
    return
  fi

  tmp=`cat /sys/class/graphics/fb0/overlays`
  if [ "$tmp" == "0" ]; then
    # tv was on overlay
    layer=0
  else
    layer=1
  fi
}

is_good_tv_type()
{
  if [ "$1" == "pal" -o "$1" == "ntsc" ]; then
    return 0
  fi
  echo "bad tv_type: $1"
  false
}

is_good_conn_type()
{
  if [ "$1" == "svideo" -o "$1" == "composite" ]; then
    return 0
  fi
  echo "bad conn_type: $1"
  false
}

is_good_layer()
{
  if [ "$1" == "0" -o "$1" == "1" ]; then
    return 0
  fi
  echo "bad layer: $1"
  false
}

# read defaults from user profile
if test -f "$profiles"; then
  defprofile=`grep -A1 '^Last written' "$profiles" | tail -n 1`
  if test -n "$defprofile"; then
    parse_profile()
    {
      # enabled
      # encoding
      is_good_tv_type $2 && tv_type=$2
      # connection
      is_good_conn_type $3 && conn_type=$3
      # layer
      is_good_layer $4 && layer=$4
      # x,y
      test -n "$5" -a -n "$6" && eval ${tv_type}_pos=$5,$6
      # w,h
      test -n "$7" -a -n "$8" && eval ${tv_type}_size=$7,$8
    }

    parse_profile $defprofile
    echo "loaded defaults from last used profile"
  fi
fi

# parse args
while true; do
  case $1 in
    "-d")
      disable_it
      exit 0
      ;;
    "-t")
      shift
      tv_type=$1
      ;;
    "-c")
      shift
      conn_type=$1
      ;;
    "-l")
      shift
      if [ "$1" == "o" ]; then
        # "other layer" - switch to different layer
        layer_toggle
      else
        layer=$1
      fi
      ;;
    "-ps")
      shift
      if [ "$1" != "0,0" ]; then
        pal_size=$1
      else
        echo "warning: ignored pal_size: $1"
      fi
      ;;
    "-pp")
      shift
      pal_pos=$1
      ;;
    "-ns")
      shift
      if [ "$1" != "0,0" ]; then
        ntsc_size=$1
      else
        echo "warning: ignored ntsc_size: $1"
      fi
      ;;
    "-np")
      shift
      ntsc_pos=$1
      ;;
    "")
      ;;
    *)
      usage
      ;;
  esac

  if ! shift; then break; fi
done

if ! is_good_tv_type $tv_type; then
  usage
fi

if ! is_good_conn_type $conn_type; then
  usage
fi

if ! is_good_layer $layer; then
  usage
fi

enable_it
